<?php
// $Id: countries.admin.inc,v 1.2 2009/10/11 06:35:07 aland Exp $

/**
 * @file
 * Admin page callbacks for the Countries module.
 */

/**
 * Menu callback; Displays a list of all countries.
 */
function countries_admin_overview() {
  $header = array();
  $header[] = array('data' => t('Name'), 'field' => 'c.name', 'sort' => 'asc');
  $header[] = array('data' => t('ISO2'), 'field' => 'c.iso2');
  $columns = variable_get('countries_admin_overview_columns', array('iso3' => t('ISO3'), 'numcode' => t('Number code'), 'continent' => t('Continent'), 'official_name' => t('Official name')));
  foreach ($columns as $key => $title) {
    $header[] = array('data' => $title, 'field' => 'c.' . $key);
  }
  $header[] = array('data' => t('Status'), 'field' => 'c.enabled');
  $header[] = array('data' => t('Operations'));

  $destination = drupal_get_destination();
  $continents = countries_get_continents();

  countries_load_iso_definitions();
  $core_countries = _country_get_predefined_list();

// Drupal 7
//  $query = db_select('countries_country', 'c')->extend('PagerDefault')->extend('TableSort');
//  $result = $query
//    ->fields('c')
//    ->orderByHeader($header)
//    ->limit(50)
//    ->execute();
  $result = pager_query('SELECT * FROM {countries_country} c '. tablesort_sql($header), 50);

  $rows = array();
  while ($country = db_fetch_object($result)) {
//  foreach ($result as $country) {
    $row = array();
    $row[] = l($country->name, COUNTRIES_ADMIN_PATH . '/' . $country->iso2, array('query' => $destination));
    $row[] = $country->iso2;
    foreach ($columns as $key => $title) {
      switch ($key) {
        case 'continent':
          $row[] = isset($continents[$country->continent]) ? $continents[$country->continent] : t('Unknown');
          break;
        case 'numcode':
          $row[] = theme('countries_number', array('country' => $country));
          break;
        case 'iso3':
        default:
          $row[] = $country->$key;
          break;
      }
    }
    $row[] = $country->enabled ? t('Enabled') : t('Disabled');
    $operations = l(t('edit'), COUNTRIES_ADMIN_PATH . '/' . $country->iso2 , array('query' => $destination));
    if (!array_key_exists($country->iso2, $core_countries)) {
      $operations .= '&nbsp;&nbsp;&nbsp;' . l(t('delete'), COUNTRIES_ADMIN_PATH . '/' . $country->iso2 . '/delete', array('query' => $destination));
    }
    $row[] = $operations;
    $rows[] = $row;
  }
  if (empty($rows)) {
    $rows[] = array(
      array('data' => t('No countries are available.'), 'colspan' => count($header)),
    );
  }
  return theme('table', $header, $rows) . theme('pager', array(), 50);
  $build['countries_table']  = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
  );
  $build['countries_pager'] = array('#theme' => 'pager');

  return $build;
}

/**
 * Menu callback; Display a country form.
 */
function countries_admin_page($country = NULL) {
  if (!isset($country)) {
    drupal_set_title(t('Add country'));
    //drupal_set_title(t('Add country'), PASS_THROUGH);
    $country = (object)array('iso2' => '',
        'iso3' => '',
        'name' => '',
        'official_name' => '',
        'continent' => 'UN',
        'enabled' => 1,
        'numcode' => 0);

  }
  return drupal_get_form('countries_admin_form', $country);
}

/**
 * Generate a country form.
 *
 * @ingroup forms
 * @see countries_admin_form_validate()
 * @see countries_admin_form_submit()
 */
// function countries_admin_form($form, &$form_state, $country) {
function countries_admin_form($form_state, $country) {
  $form = array();
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#default_value' => $country->name,
    '#description' => t('Specify a unique name for this country.'),
    '#required' => TRUE,
    '#maxlength' => 95,
  );
  $iso2_editable = TRUE;
  if (!empty($country->iso2)) {
    countries_load_iso_definitions();
    $core_countries = _country_get_predefined_list();
    $iso2_editable = ! array_key_exists($country->iso2, $core_countries);
  }
  if ($iso2_editable) {
    $form['iso2'] = array(
      '#type' => 'textfield',
      '#title' => t('ISO2 code'),
      '#default_value' => $country->iso2,
      '#description' => t('Specify a unique ISO2 code for this country. This is used as the key to this country, changing it may result in the loss of data.'),
      '#required' => TRUE,
      '#maxlength' => 2,
    );
  }
  else {
    $form['iso2'] = array(
      '#type' => 'value',
      '#value' => $country->iso2,
    );
    $form['iso2_info'] = array(
      '#type' => 'textfield',
      '#title' => t('ISO2 code'),
      '#value' => $country->iso2,
      '#disabled' => TRUE,
      '#description' => t('Core country ISO2 codes are not editable.'),
    );
  }
  $form['iso3'] = array(
    '#type' => 'textfield',
    '#title' => t('ISO3 code'),
    '#default_value' => $country->iso3,
    '#description' => t('Specify a unique ISO3 code for this country.'),
    '#required' => FALSE,
    '#maxlength' => 3,
  );
  $form['official_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Official name'),
    '#default_value' => $country->official_name,
    '#description' => t('Specify the unique official name for this country.'),
    '#required' => FALSE,
    '#maxlength' => 127,
  );
  $form['numcode'] = array(
    '#type' => 'textfield',
    '#title' => t('Number code'),
    '#default_value' => empty($country->numcode) ? '' : $country->numcode,
    '#description' => t('Specify a unique number code for this country.'),
    '#required' => FALSE,
    '#maxlength' => 5,
  );

  $form['continent'] = array('#type' => 'select',
    '#title' => t('Continent'),
    '#options' => countries_get_continents(),
    '#default_value' => $country->continent,
    '#required' => TRUE,
  );
  $form['enabled'] = array('#type' => 'checkbox',
    '#title' => t('Enabled'),
    '#default_value' => $country->enabled,
  );

  if (!empty($country->iso2)) {
    $form['#country'] = $country;
  }
  $form['submit'] = array('#type' => 'submit', '#value' => t('Save'));

  return $form;
}

/**
 * Validate country form submissions.
 */
function countries_admin_form_validate($form, &$form_state) {
  // Edits
  $unique_keys = array('name', 'iso2', 'iso3', 'official_name', 'numcode');

  foreach ($unique_keys as $key) {
    if (drupal_strlen($form_state['values'][$key])) {
      $iso2 = isset($form['#country']) ? $form['#country']->iso2 : FALSE;
      if ($key == 'iso2' || $key == 'iso3') {
        $form_state['values'][$key] = drupal_strtoupper($form_state['values'][$key]);
      }
      else {
        $form_state['values'][$key] = trim($form_state['values'][$key]);
      }
      switch($key) {
        case 'name':
          break;
        case 'iso2':
        case 'iso3':
          if (preg_match('/[^A-Z]/', $form_state['values'][$key])) {
            form_set_error($key, t('Please only use uppercase characters for ISO codes.'));
            continue;
          }
          else {
            $length = $key == 'iso2' ? 2 : 3;
            if (drupal_strlen($form_state['values'][$key]) != $length) {
              form_set_error($key, t('The ISO!length code must be exactly !length characters.', array('!length' => $length)));
              continue;
            }
          }
          break;
        case 'official_name':
          break;
        case 'numcode':
          if (preg_match('/[^0-9]/', $form_state['values'][$key])) {
            form_set_error($key, t('Only numbers are allowed for the number code.'));
            continue;
          }
          break;
      }
      if ($key == 'numcode' && empty($form_state['values'][$key])) {
        // Duplicate 0 values are OK.
      }
      // Drupal 7
      // elseif ((bool) db_query_range("SELECT 1 FROM {countries_country} WHERE iso2 <> :orginaliso2 AND LOWER({$key}) = LOWER(:$key)", 0, 1, array(':orginaliso2' => $iso2, ":$key" => $form_state['values'][$key]))->fetchField()) {
      elseif (db_result(db_query_range("SELECT 1 FROM {countries_country} WHERE iso2 <> '%s' AND LOWER({$key}) = LOWER('%s')", 0, 1, array($iso2, $form_state['values'][$key])))) {
        form_set_error($key, t('The value %value is already taken.', array('%value' => $form_state['values'][$key])));
      }
    }
  }
}

/**
 * Process country form submissions.
 */
function countries_admin_form_submit($form, &$form_state) {
  $country = (object) $form_state['values'];
  $iso2 = isset($form['#country']) ? $form['#country']->iso2 : FALSE;

  if ($iso2) {
    if ($country != $iso2) {
      db_query("UPDATE {countries_country} SET iso2 = '%s' WHERE iso2 = '%s'", $country->iso2, $iso2);
//      db_update('countries_country')
//        ->fields(array('iso2' => $country->iso2))
//        ->condition('iso2', $iso2)
//        ->execute();
    }
    drupal_write_record('countries_country', $country, array('iso2'));
    $message = t('The country %country has been updated.', array('%country' => $country->name));
    // Drupal 7
    // field_cache_clear();
    // Drupal 6
    cache_clear_all('*', content_cache_tablename(), TRUE);
  }
  else {
    drupal_write_record('countries_country', $country);
    $message = t('Added country %country.', array('%country' => $country->name));
  }
  // TODO: I18n
  // countries_locale_process($country);
  drupal_set_message($message);
  $form_state['redirect'] = COUNTRIES_ADMIN_PATH;

}

/**
 * Menu callback; confirm deletion of a country.
 *
 * @ingroup forms
 * @see countries_admin_delete_submit()
 */
//function countries_admin_delete($form, &$form_state, $country) {
function countries_admin_delete($form_state, $country) {
  countries_load_iso_definitions();
  $core_countries = _country_get_predefined_list();
  if (array_key_exists($country->iso2, $core_countries)) {
    drupal_set_message(t('Core countries defined by the system can not be deleted.'), 'error');
    drupal_goto(COUNTRIES_ADMIN_PATH);
  }
  $form = array();
  $form['#country'] = $country;

  return confirm_form($form,
    t('Are you sure you want to delete the country %country?', array('%country' => $country->name)),
    COUNTRIES_ADMIN_PATH,
    t('References that use this country will become invalid. This action cannot be undone.'),
    t('Delete'),
    t('Cancel')
  );
}

/**
 * Process countries delete form submission.
 */
function countries_admin_delete_submit($form, &$form_state) {
  $country = $form['#country'];
  db_query("DELETE FROM {countries_country} WHERE iso2 = '%s'", $country->iso2);
//  db_delete('countries_country')
//    ->condition('iso2', $country->iso2)
//    ->execute();
  drupal_set_message(t('Deleted country %country.', array('%country' => $country->name)));
  $form_state['redirect'] = COUNTRIES_ADMIN_PATH;
}
